using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;

namespace MouseGestures
{
    public partial class Form1 : Form
    {
        StringBuilder symbols = new StringBuilder();
        Point previous;
        DateTime previousTime = DateTime.Now;

        public Form1()
        {
            InitializeComponent();
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            base.OnMouseDown(e);

            symbols.Remove(0, symbols.Length);
            previous = e.Location;
            previousTime = DateTime.Now; // This is a pretty rough timer but is ok for our purposes.
        }

        protected override void OnKeyPress(KeyPressEventArgs e)
        {
            base.OnKeyPress(e);
        }

        protected override void OnMouseMove(MouseEventArgs e)
        {
            base.OnMouseMove(e);
            if (e.Button == MouseButtons.Left)
            {
                int dx = e.X - previous.X;
                int dy = -(e.Y - previous.Y); // screen y is the other way round
                DateTime now = DateTime.Now;
                double time = (now-previousTime).TotalSeconds;
                if (time < 10e-6) // hack to prevent division by zero
                {
                    time = 0.01; // typical time interval
                }

                // mouse velocity in pixels per second
                double vx = dx / time;
                double vy = dy / time;

                char n = 'X';
                // velocity of less than 200 pixels per second, independent of the direction
                if (vx * vx + vy * vy < 200.0 * 200.0)
                {
                    n = '0';
                }
                else
                {
                    double phi = Math.Atan2(vy, vx);
                    if (phi > 0.75 * Math.PI)
                        n = 'W';
                    else if (phi > 0.25 * Math.PI)
                        n = 'N';
                    else if (phi > -0.25 * Math.PI)
                        n = 'E';
                    else if (phi > -0.75 * Math.PI)
                        n = 'S';
                    else
                        n = 'W';
                }
                symbols.Append(n);

                previous = e.Location;
                previousTime = now;
            }
        }

        protected override void OnMouseUp(MouseEventArgs e)
        {
            base.OnMouseUp(e);

            String s = symbols.ToString();
            if (Regex.IsMatch(s, "^(0*N){3,}0*(0*E){3,}0*(0*S){3,}0*(0*W){3,}0*(0*N)*0*$"))
            {
                Text = "circle to the right";
            }
            else if (Regex.IsMatch(s, "^(0*S){3,}(0*N){3,}0*$"))
            {
                Text = "down and up";
            }
            else if (Regex.IsMatch(s, "^(0*N){3,}(0*E){3,}(0*S){3,}(0*W){3,}(0*E){3,}(0*S){3,}0*$"))
            {
                Text = "letter R from its lowest left point";
            }
            else
            {
                Text = "???";
            }
        }
    }
}